package Site::FieldSQL;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use PObject;
@ISA = qw( PObject );

# Needs this to escape data strings
use HTML::Entities;
use Carp;
use strict;

$Site::FieldSQL::AUTOSETTINGS = 1;
%Site::FieldSQL::AUTOFIELDS = ( TYPE => "field",
				     NAME => undef,
				     LABEL => undef,
				     VALUE => undef,
				     LOCKED => undef,
				     BOUND => 1,
				     HREF => undef,
				     HEIGHT => 3,
				     WIDTH => 30,
				     DEFAULT => "",
				     FORMAT => '%s',
	                             DISABLED => 0,
	                             VISIBLE => 1,
                                     RULE => "",
				     WRAP => undef,
	                             VALIDATIONTEXT => "Failed" ,
	                             ROWSOURCETYPE => "list", 
	                             ROWSOURCEDELIMITER => "<BR>\n",
				     ROWSOURCEVALUES => undef,
                                     FORMNAME => undef,
                                     SUPPRESSALL => 0, # Special for TABLES right now, going to go away
                                     ROWSOURCEQUERY => undef );

# This is the field package for the WebFormSQL package
sub OBJ_NEW
{
    my $Self = shift;

    if( ! defined( $Self -> { ROWSOURCEVALUES } ) )
    {
    	$Self -> { ROWSOURCEVALUES } = {};
    }

    $Self -> { LASTVALUE } = "UNSET";
    $Self -> { RUNNINGTOTAL } = 0;
    
    # The attributes are just hash settings ( setting => 1 )
    $Self -> { ATTRIBUTES } = {};

    # This is set to TRUE is there is a value already set on the field close,
    # the value can not be reset after that
    #$Self -> { LOCKED } = 0;
    
    # Return me
    return $Self;
}

sub OBJ_HOOK_VALUE
{
    # If I'm not locked just pass it on to my base
    my $Self = shift;

    $Self -> _dmesg( "Entering VALUE override sub for locking", 3 );
    if( $Self -> Locked() )
    {
	$Self -> _dmesg( "Field value locked, no update permitted", 3 );
	return( undef );
    }
    else
    {
	$Self -> _dmesg( "Field not yet locked, passing value", 3 );
	return( shift() );
    }
}

sub SetDatabase
{
    # Takes a DBH as an arg
    my $Self = shift;
    $Self -> { DBH } = shift;
}

# Generates the field part (interactive) of the field
sub GenerateFieldHTML
{
    my $Self = shift;
    my $RecordRef = shift; # This is a hash ptr to possible {} vals in a href

    my $t = $Self -> Type;
    my( $href );

    if( $Self -> Disabled() )
    {
	$t = "disabled";
    }

    if( ! $Self -> Visible() )
    {
	$t = "hidden";
    }

    if( $Self -> Type() eq "runningtotal" )
    {
	# Add last value in, set to current value, flag as updated
	if( $Self -> { LASTVALUE } ne $Self -> Value() )
	{
	    $Self -> { LASTVALUE } = $Self -> Value();
	    $Self -> { RUNNINGTOTAL } += $Self -> Value();
	    $Self -> Value( $Self -> { RUNNINGTOTAL } );

	    # Set the variable NAME_RUNNINGTOTAL
	    #XAD::Variables -> Set( $Self -> VarName() . "_runningtotal", $Self -> { RUNNINGTOTAL } );
	}

	$t = "disabled";
    }
    
    my $DisplayValue = sprintf( $Self -> Format(), $Self -> Value() );

    my( $RetString, $TmpString, $sth, $AryRef );

    $Self -> _dmesg( "Generating HTML field for field: '" . 
		     $Self -> VarName() . 
		     "'", 2 );
    $Self -> _dmesg( "Generating HTML field for value: '" . 
		     $DisplayValue . 
		     "'", 2 );

    # Build data
    $Self -> _BuildListData();

    # Make the field
    if( $t eq "field" )
    {
	# Standard field
	$RetString = '<INPUT TYPE=TEXT NAME=' . $Self -> VarName() . ' VALUE="' . encode_entities( $DisplayValue ) . '">';
    } elsif( $t eq "textarea" )
    {
	# Text area
	$RetString = "<TEXTAREA NAME=". $Self -> VarName();
	$RetString .= ' ROWS=' . $Self ->  Height();
	$RetString .= ' COLS=' . $Self ->  Width();
	if( $Self -> Wrap() )
	{
		$RetString .= ' WRAP=' . $Self -> Wrap();
	}
	$RetString .= '>' . encode_entities( $DisplayValue ) . '</TEXTAREA>';
    } elsif( $t eq "checkbox" )
    {
	$RetString = '<INPUT TYPE=CHECKBOX NAME=' . $Self -> VarName();
	$RetString .= ' VALUE=1';
	if( $DisplayValue ||
	    ( lc( $DisplayValue ) eq "true" ) ||
	    ( lc( $DisplayValue ) eq "t" ) )
	{
	    $RetString .= " CHECKED";
	}
	$RetString .= '>';
    } elsif( $t eq "optiongroup" )
    {
	$RetString = join( $Self -> RowSourceDelimiter(),
			   $Self -> _GenerateFormList( '<INPUT TYPE=RADIO NAME=' .
						       $Self -> VarName() .
						       ' VALUE=', '>',
						       "CHECKED" ) );
    } elsif( $t eq "dropbox" )
    {
	$RetString = "<SELECT NAME=" . $Self -> VarName() . ">";
	$RetString .= join( "\n", $Self -> _GenerateFormList( '<OPTION VALUE=', '>', "SELECTED" ) );
	$RetString .= "</SELECT>";
    }elsif( $t eq "disabled" )
    {
	# Standard field, if href then <A it too

	# May or may not really be disabled.. Let's check
	if( $Self -> Type() eq "checkbox" )
	{
	    # Should show yes/no
	    if( $DisplayValue )
	    {
		$RetString = "Yes";
	    }
	    else
	    {
		$RetString = "No";
	    }
	}elsif( $Self -> Type() eq "dropbox" )
	{
	    # Need to look up the value from a query...?
	    if( $Self -> RowSourceType eq "query" )
	    {
		# TODO!!
		    
	    }
	}
	else
	{
	    # Just spit it out
	    $RetString = encode_entities( $DisplayValue );
	}

	if( defined( $Self -> Href() ) )
	{
	    $href = $Self -> Href();
	    $href =~ s/{(.*?)}/$RecordRef -> {$1}/ge;
	    $RetString = '<A HREF="' . $href . '">' .
		$RetString . "</A>";
	}

	if( ! $Self -> SuppressAll() )
	{
	    $RetString .= '<INPUT TYPE=HIDDEN NAME=' . $Self -> VarName()
		. ' VALUE="' . encode_entities( $DisplayValue ) . '">';
	}
    }elsif( $t eq "hidden" )
    {
	# Just invisible
	$RetString = '<INPUT TYPE=HIDDEN NAME=' . $Self -> VarName . ' VALUE="' . encode_entities( $DisplayValue ) . '">';
    } elsif( $t eq "carry" )
    {
	$RetString = '<INPUT TYPE=HIDDEN NAME=' . $Self -> Name . ' VALUE="' . encode_entities( $DisplayValue ) . '">';
    } else	
    {
	$RetString = "Unknown field type: " . $t;
    }

    if( ! $Self -> SuppressAll() )
    {
	# Also embed a HIDDEN prev-value of the field
	$RetString .= '<INPUT TYPE=HIDDEN NAME=' . $Self -> VarName()
	    . '_ VALUE="' . encode_entities( $DisplayValue ) . '">';
    }

    return( $RetString );
}

# Generates the label part (static) of the field
sub GenerateLabelHTML
{
    my $Self = shift;

    if( ( ! $Self -> Visible() ) ||
	( $Self -> Type() eq "carry" ) )
    {
	return( "" );
    }

    if( $Self -> Label() )
    {
	return( encode_entities( $Self -> Label() ) );
    }
    else
    {
	return( encode_entities( $Self -> Name() ) );
    }
}

sub GenerateFieldForSelectSQL
{
    my $Self = shift;

    # Returns the SQL field name
    if( ( ! $Self -> Disabled() ) && ( $Self -> Type() ne "carry" ) )
    {
	if( ( $Self -> Type() eq "dropbox" ) || ( $Self -> Type() eq "optiongroup" ) ) 
	{
	    # Make sure we're not a list
	    if( $Self -> RowSourceType() ne "list" )
	    {
		return( $Self -> Name );
	    }
	}
	else
	{
	    return( $Self -> Name );
	}
    }

	if( $Self -> Type() ne "carry" )
	{
    return( $Self -> Name() );
	}
	else
	{
		return( undef );
	}
}

sub GenerateFieldForUpdateSQL
{
    my $Self = shift;

    # Returns the SQL field name
    if( ( ! $Self -> Disabled() ) && ( $Self -> Type() ne "carry" ) )
    {
	if( ( $Self -> Type() eq "dropbox" ) || ( $Self -> Type() eq "optiongroup" ) ) 
	{
	    # Make sure we're not a list
	    #if( $Self -> RowSourceType() ne "list" )
	    #{
		return( $Self -> Name );
	    #}
	}
	else
	{
	    return( $Self -> Name );
	}
    }

    return( undef );
}

# Variable attribs
# type            Specifies control type:
#                   field (default)
#                   dropbox
#                   optiongroup
#                   checkbox
#


# Valid field attributes and their meanings (default):
# format          sprintf format for field (%s)
# disabled        displays but does not allow edit (false)
# visible         if field is visible or hidden (true)
# default         default value for add forms (undef)
#
# validation-rule Rule to validate against, >,<.,==, etc (undef)
# validation-text displayed if validation fails (failed)
#
# These apply to dropbox only:
# row-source-type   Type of row source, query/table, list (list)
# row-source-query  Query. (undef)
# row-source-values Hash.  Filled in automatically if row-source-type is
#                   a query, otherwise needs to be filled before created.
#                   key=returned value=displayed
# row-source-delimiter Put between values of option groups (<BR>)
#
# If row-source is a query then values returned will be column 0,
# the rest of the columns will be cat'd together to form the displayed
# value separated by spaces.


sub __attribute
{
    # This sets attribs ( ATTRIB, value )
    # This gets the attrib ( ATTRIB )
    my $Self = shift;
    my $AttribName = shift;

    if( @_ != 0 )
    {
	$Self -> { ATTRIBUTES } -> { lc( $AttribName ) } = shift;
	$Self -> _dmesg( "Setting $AttribName to '" . $Self -> { ATTRIBUTES } -> { lc( $AttribName ) } . "'", 2 );
    }

    return( $Self -> { ATTRIBUTES } -> { lc( $AttribName ) } );
}

# The following are just for internal use.  Yeah right. ;-)
sub _GenerateFormList
{
    # Takes the pre,mid, and postfix.  Assembles like this:
    # prefix . VALUE . midfix . DISP
    my $Self = shift;

    my $Prefix = shift;
    my $Midfix = shift;
    my $Selected = shift;

    my $DisplayValue = $Self -> _FormatValue( $Self -> Value() );
    my( $TmpString, $ThisValue );

    $Self -> _dmesg( "Processing quick list gen", 2 );

    # Returns array of completed rows
    return( map
	    {
		$ThisValue = $Self -> _FormatValue( $_ );

		$Self -> _dmesg( "This is:" . $ThisValue, 2 );

		$TmpString = $Prefix . '"' . encode_entities( $ThisValue ) . '"';
		if( $ThisValue eq $DisplayValue )
		{
		    $TmpString .= " " . $Selected;
		}

		$TmpString .= $Midfix;

 		$TmpString .= encode_entities( ${ $Self -> { ROWSOURCEVALUES } }{ $_ } );
	}  keys( %{ $Self -> { ROWSOURCEVALUES } } ) );
}

# This is an alias for validationtext, eventually vali..text will go away.
# it's too freakin hard to type all the time.
sub __ruletext
{
    my $Self = shift;

    $Self -> ValidationText( @_ );
}

sub _FormatValue
{
    my $Self = shift;
    my $Value = shift;

    return( sprintf( $Self -> Format(), $Value ) );
}

sub _BuildListData
{
    # This builds the list if SQL table data is requested as a field
    my $Self = shift;

    my( %ListData, $key );
    my( $sth, @rowdata );

    if( $Self -> RowSourceType() eq "query" )
    {
	# Execute this query and create a hash
	$Self -> _dmesg( "Creating data from query: " . $Self -> RowSourceQuery(), 3 );
	$sth = $Self -> { DBH } -> prepare( $Self -> RowSourceQuery() );

	if( ! defined( $sth ) )
	{
	    # Error!!
	    print "Content-type: text/plain\n\n";
	    print "An SQL error occurred while processing:\n";
	    print $Self -> RowSourceQuery() . "\n";
	    print $Self -> { DBH } -> errstr . "\n";
	    confess( "I died" );
	}

	$sth -> execute();
	while( @rowdata = $sth -> fetchrow_array() )
	{
	    $key = shift( @rowdata );
	    $ListData{ $key } = join( " ", @rowdata );
	}

	# Put it in me
    	$Self -> { ROWSOURCEVALUES } = \%ListData;
    }
}

sub IsValid
{
    # Return 0 or 1
    # Check through my rule.  Each phrase is of
    # >,<,=,!=,eq,ne VALUE
    # And can be compounded by AND or OR
    my $Self = shift;
    
    my $OK = 1;
    my $ThisOK;
    
    my $Mode = "AND";
    
    my $Counter = 0;
    my( $Op, $Value );
    
    my $CurrVal = $Self -> Value();
    
    # First break apart the rules
    $Self -> { RULE } =~ s/and/AND/ig;
    $Self -> { RULE } =~ s/or/OR/ig;
	
    for ( split( /(AND|OR)/, $Self -> Rule() ) )
    {
	if( $Counter % 2 == 0 )
	{
	    # Even, since starting at zero, is a phrase
	    ( $Op, $Value ) = $_ =~ /^\s*([<>=]|!=|ne|eq)\s*(.*)/;

	    # Clean up the value
	    $Value =~ s/^\"?(.*?)\"?\s*/$1/;
	    
	    $Self -> _dmesg( "Op: " . $Op, 3 );
	    $Self -> _dmesg( "Val: " . $Value, 3 );
	    $Self -> _dmesg( "CurrVal: " . $CurrVal, 3 );
	    
	    # Assume OK
	    $ThisOK = 1;
	    if( $Op eq "<" )
	    {
		$ThisOK = ( $CurrVal < $Value );
	    } elsif( $Op eq ">" )
	    {
		$ThisOK = ( $CurrVal > $Value );
	    } elsif( $Op eq "=" )
	    {
		$ThisOK = ( $CurrVal == $Value );
	    } elsif( $Op eq "!=" )
	    {
		$ThisOK = ( $CurrVal != $Value );
	    } elsif( $Op eq "eq" )
	    {
		$ThisOK = ( $CurrVal eq $Value );
	    } elsif( $Op eq "ne" )
	    {
		$ThisOK = ( $CurrVal ne $Value );
	    } else
	    {
		confess( "Unknown op: " . $Op );
	    }

	    $Self -> _dmesg( "ThisOK is " . $ThisOK, 3 );
	    
	    # Put our OK's together
	    if( $Mode eq "AND" )
	    {
		$OK = $OK && $ThisOK;
	    }
	    else
	    {
		$OK = $OK || $ThisOK;
	    }
	    
	    $Self -> _dmesg( "Combine type is " . $Mode, 3 );
	    $OK = sprintf( "%01d", $OK );
	    $Self -> _dmesg( "OK is " . $OK, 3 );
	}
	else
	{
	    # Odd is a modifier
	    $Mode = $_;
	}

	$Counter++;
    }
    
    return( $OK );
}

sub displayError
{
    my $Self = shift;
    
    $Self -> _dmesg( "Displaying error: " . $Self -> ValidationText(), 3 );
    
    return( $Self -> GenerateLabelHTML() . ": " . $Self -> ValidationText() . "<BR>\n" );
}

sub SetDefault
{
    my $Self = shift;

    # Clear lastval
    $Self -> { LASTVALUE } = "UNSET";
    $Self -> { RUNNINGTOTAL } = 0;
    
    $Self -> _dmesg( "Setting " . $Self -> Name() . " to '" . $Self -> Default() . "'", 3 );
    $Self -> SetValue( $Self -> Default() );
}

sub SetValue
{
    # This sets the value even if it's undef, UNLESS IT'S LOCKED
    my $Self = shift;

    if( ( $Self -> Locked() ) ||
	( ! $Self -> Bound() ) )
    {
	$Self -> _dmesg( "Unconditional setvalue bypassed, FIELD LOCKED", 7 );
    }
    else
    {
	$Self -> _dmesg( "Unconditional setvalue", 7 );
	$Self -> { VALUE } = shift;
    }
}

sub SqlValue
{
    # Return the SQL value of the form
    my $Self = shift;
    my $t = $Self -> Value();

    if( $Self -> Type() eq "checkbox" )
    {
	$Self -> _dmesg( "Checkbox made value: " . $t, 1 );
	$t = lc( $t );
	if( ( $t eq "on" ) ||
	    ( $t eq "1" ) ||
	    ( $t eq "true" ) ||
	    ( $t eq "t" ) )
	{
	    return( 'true' );
	}
	else
	{
	    return( 'false' );
	}
    }
    else
    {
	# Just passthru
	return( $t );
    }
}

sub VarName
{
    # Returns a name suitable for a form variable name
    my $Self = shift;

    if( defined( $Self -> FormName() ) )
    {
	return( $Self -> FormName() . "." . $Self -> Name() );
    }
    else
    {
	return( $Self -> Name() );
    }
}

# Disable HREF subs
#XAD::ScreenBase -> NoSub( "href" );
#XAD::ScreenBase -> NoSub( "DEFAULT" );
# This breaks shit too...

# Gotta return true
1;
