package Site::FormSQL;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

=head1 NAME

FormSQL - SQL web forms module

=head1 SYNOPSIS

	use FormSQL;

	$f = FormSQL -> new();
	$f -> Table( "test" );
	$f -> DBH( $dbh );
	$f -> PKey( "testid" );
	$f -> Opts( Site::Page -> Options() );

	$f -> addfield( {
				name => 'field1',
				label => 'Field 1'
		} );
	$f -> HandleData();
	$f -> display();

=head1 DESCRIPTION

The FormSQL module is deprecated in favor of the newer SQLForm module.

The FormSQL module allows relatively easy construction of SQL bound
forms and fields.  It allows editing, inserting, and deleting of SQL
rows by passing GET and POST methods to the CGI page that contains
them.
=head1 AUTHOR

Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.

This program is free software  and is provided "as is" without
express or implied warranty. You can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

use PObject;
@ISA = qw( PObject );

use strict;

use HTML::Entities;
use Carp;

use Site::FieldSQL;

$Site::FormSQL::AUTOSETTINGS = 1;
%Site::FormSQL::AUTOFIELDS = ( ACTION => $::ENV{ REQUEST_URI },
				    FORMREF => undef,
				    DATABASE => undef,
				    TABLE => undef,
				    PKEY => undef,
				    ONDELETE => "Record deleted",
				    ASKDELETE => "Delete record?",
				    ALLOWDELETE => 1,
				    
				    EDITBUTTON => 'EDIT',
				    ADDBUTTON => 'ADD',
				    DELETEBUTTON => 'DELETE',

# Hooks are subroutine refs.  They will be passed a hashref
# for the record to be added/edited/deleted, they should
# return "undef" if everything's OK, or a message if not
				    HOOKDELETE => undef,
				    HOOKADD => undef,
				    HOOKEDIT => undef );
				    

# This is the field package for the WebFormSQL package
sub OBJ_NEW
{
    my $Self = shift;

    $Self -> { ACTION } =~ s/\?.*//;
    $Self -> { BADFORM } = 0;

    $Self -> { SCREEN } = undef;
    
    # Fields are a big array
    $Self -> { FIELDS } = [];

    $Self -> { SHOWMESSAGE } = "";
    
    $Self -> { displayVALUE } = undef;
    # Set to true if updated
    $Self -> { DIDUPDATE } = 0;
    

    # Return me
    return $Self;
}

sub Name
{
	my $Self = shift;

	return( join( ".", "form",
				$Self -> Table(),
				$Self -> PKey() ) );
}

# Generates the entire HTML form
sub display
{
	my $Self = shift;

	# The form vars are here
	my $FormRef = $Self -> FormRef();
	#my %FormValues = %{ $FormRef };

	# Set some vars for convenience	
	my $Action = $Self -> Action();
	my $ButtonValue = $Self -> AddButton();
	my $AddOrEdit = 'ADD';
	
	# Other vars used in me
	my( $Field, $Statement, $RecordRef, $sth, $DispAsk );

	my $EditMode = 0;
	my $Name = $Self -> Name();
	my $PKey = $Self -> PKey();

	# Integrated form handler
	#$Self -> _dmesg( "Executing SQL", 3 );
	#$Self -> { DIDUPDATE } =  $Self -> HandleFormSQL();

	#if( $Self -> { DIDUPDATE } && defined( $Self -> OnExecGoto() ) )
	#{
	#    $Self -> GetXadObject() -> Parent() -> Object() -> redirect( $Self -> OnExecGoto() );
	#    return( undef );
	#}
	
	# Check if deleting rec ( doesn't work well yet.. )
	if( ( $FormRef -> { __button } eq $Self -> DeleteButton() ) &&
	 ( $FormRef -> { __xad_formname } eq $Self -> Name() ) )
	{
	    if( $Self -> AllowDelete() )
	    {
		# This should get out all the fields from the form and carry them in to
		# the next screen, maybe someday?
		$Statement = "select ";
		$Statement .= join( ", ", $Self -> Pkey(),
				    map( $_ -> GenerateFieldForSelectSQL(),
					 @{ $Self -> { FIELDS } } ) );
		$Statement .= " from " . $Self -> Table();
		$Statement .= " where " . $Self -> PKey() . "=?";
		
		$sth = $Self -> { DATABASE } -> prepare( $Statement );
		
		if( ! defined( $sth ) )
		{
		    # Error!!
		    print "Content-type: text/plain\n\n";
		    print "An SQL error occurred while processing:\n";
		    print $Self -> RowSourceQuery() . "\n";
		    print $Self -> { DATABASE } -> errstr . "\n";
		    confess( "I died" );
		}
		
		$sth -> execute( $FormRef -> { $Self -> VarPKey() } );
		$RecordRef = $sth -> fetchrow_hashref();
		
		#for ( keys( %{ $RecordRef } ) )
		#{
		#  XAD::Variables -> Set( $_, $RecordRef -> { $_ } );
		#}	    
		
		if( $Self -> HookDelete() )
		{
			# Call the delete hook
			$DispAsk = &{ $Self -> HookDelete() }( $RecordRef );
			if( $DispAsk )
			{
				$Self -> _Add( $DispAsk );
				return( undef );
			}
		}

		# Show our message
		$DispAsk = $Self -> AskDelete();
		$DispAsk =~ s/{(.*?)}/$RecordRef -> {$1}/ge;
		$Self -> _Add( $DispAsk );
		$Self -> _Add( '<FORM ACTION=' . $Action . ' METHOD=POST>' . "\n" );
		$Self -> _Add( '<INPUT TYPE=HIDDEN NAME=' .
			       $Self -> VarPKey() . ' VALUE="' . $FormRef -> { $Self -> VarPKey() } . '">' . "\n" );


		for ( keys( %{ $RecordRef } ) )
		{
		    $Self -> _Add( '<INPUT TYPE=HIDDEN NAME=' .
				   $_ .
				   ' VALUE="' .
				   encode_entities( $RecordRef -> { $_ } ) .
				   '">' . "\n" );	      
		}	    
		
		$Self -> _Add( '<INPUT TYPE=HIDDEN NAME=ACTION VALUE="DELETE.FINAL">' . "\n" );
		$Self -> _Add( '<INPUT TYPE=HIDDEN NAME=__xad_formname VALUE="' . $Self -> Name() . '">' );
		$Self -> _Add( '<INPUT TYPE=SUBMIT NAME=__button VALUE=YES>' . "\n" );
		$Self -> _Add( '<INPUT TYPE=SUBMIT NAME=__button VALUE=NO>' . "\n" );
		$Self -> _Add( '</FORM>' . "\n" );
		
		return( undef );
	    }
	    else
	    {
		$Self -> _Add( "<H1>Illegal delete operation attempted</H1>" );
		return( undef );
	    }
	}
	
	# If I'm broken, remove the action handler
	# so that we believe we are not destined for
	# something else
	if( $Self -> { BADFORM } )
	{
	    # If action was EDIT, then carry over embeded pkey
	    if( $FormRef -> { action } eq "EDIT" )
	    {
		# Copy our embedded pkey in to the real named var
		$Self -> _dmesg( "Carrying over hidden PKEY for edit form", 4 );
		#XAD::Variables -> Set( $Self -> PKey() =>
		#		       $FormRef -> { $Self -> PKey() . "_pkey" } );
	    }
	    
	    $Self -> _dmesg( "Deleting ACTION because of bad form", 3 );
	    delete( $FormRef -> { action } );
	}

	# First fill in the fields if applicable
	if( exists( $FormRef -> { $Self -> PKey() } ) )
	{
	    if( ! exists( $FormRef -> { action } ) )
	    {
		$Self -> _dmesg( "Setting form to EDIT mode", 2 );
		$EditMode = 1;
		$ButtonValue = $Self -> EditButton();
		$AddOrEdit = 'EDIT';
	    }
	    else
	    {
		$Self -> _dmesg( "Form data has associated ACTION",2 );
	    }
	}
	else
	{
	    $Self -> _dmesg( $Self -> PKey() . " not set in varhash", 4 );
	}

	# If not bad form, fill in defaults
	if( ! $Self -> { BADFORM } )
	{
	    $Self -> _FieldDefaults();
	}
	else
	{
	    # If it is a bad form, show errors
	    $Self -> _DumpErrors();

	    # Also fill in what they put in
	    $Self -> _FillFields( $FormRef );
	}

	# We'll look up what's in the DB if EditMode AND not a bad form
	if( $EditMode && ( ! $Self -> { BADFORM } ) )
	{
		# Lookup fields
		$Statement = "select ";

		$Statement .= join( ", ", $Self -> PKey(), 
				grep( defined( $_ ),
				map( $_ -> GenerateFieldForSelectSQL(),
					@{ $Self -> { FIELDS } } ) ) );
		$Statement .= " from " . $Self -> Table();
		$Statement .= " where " . $Self -> PKey() . "=?";

		$Self -> _dmesg( "SQL statement: " . $Statement, 2 );

		# OK, exec and fill in the field values
		$sth = $Self -> { DATABASE } -> prepare( $Statement );

		if( ! defined( $sth ) )
		{
		    # Error!!
		    print "Content-type: text/plain\n\n";
		    print "An SQL error occurred while processing:\n";
		    print $Self -> RowSourceQuery() . "\n";
		    print $Self -> { DATABASE } -> errstr . "\n";
		    confess( "I died" );
		}

		# This is the literally passed PKEY
		$sth -> execute( $FormRef -> { $Self -> PKey() } );
		$RecordRef = $sth -> fetchrow_hashref();

		if( defined( $RecordRef ) )
		{
		    map( $_ -> SetValue( $RecordRef -> 
					 { $_ -> GenerateFieldForSelectSQL() } ),
			 @{ $Self -> { FIELDS } } );
		}
		else
		{
		    $Self -> _Add( "Record not found<BR>\n" );
		    $EditMode = 0;
		}
	}

	# Form/table header
	$Self -> _Add( $Self -> { SHOWMESSAGE } . "<BR>\n" );
	
	$Self -> _Add( '<FORM ACTION="' . $Action . '" METHOD=POST>' );
	$Self -> _Add( '<INPUT TYPE=HIDDEN NAME=__xad_formname VALUE="' . $Name . '">' );

	# If in edit mode, we need to embed the pkey we're editing
	# May want to append __something to it at some point to
	# preserve the original value in case they added the pkey as
	# an editable field
	if( $EditMode )
	{
		$Self -> _Add( '<INPUT TYPE=HIDDEN NAME=' . $Self -> VarPKey() );
		$Self -> _Add( ' VALUE="' . encode_entities( $FormRef -> { $Self -> PKey() } ) . '">' . "\n" );
	}

	$Self -> _Add( "<TABLE BORDER=0>" );

	# Here we'll loop through all the fields we own
	for $Field ( @{ $Self -> { FIELDS } } )
	{
		# Ask the field to dump it's stuff
		$Self -> _Add( "<TR>\n<TD>" );
		$Self -> _Add( $Field -> GenerateLabelHTML() );
		$Self -> _Add( "</TD>\n<TD>" );
		$Self -> _Add( $Field -> GenerateFieldHTML() );
		$Self -> _Add( "</TD>\n</TR>\n" );
	}

	# Form/table footer now

	$Self -> _Add( "</TABLE>" );
	$Self -> _Add( '<INPUT TYPE=HIDDEN NAME=ACTION VALUE="' . $AddOrEdit . '">' );
	$Self -> _Add( '<INPUT TYPE=SUBMIT VALUE="' . $ButtonValue . '">' );
	if( $EditMode && $Self -> AllowDelete() )
	{
	    $Self -> _Add( '<INPUT TYPE=SUBMIT NAME=__BUTTON VALUE="' . $Self -> DeleteButton() . '">' );
	}
	
	$Self -> _Add( '</FORM>' );
	
}

# Handles the form update based on the %hash passed.  The passed hash
# should be pair values of the returned form submission
#
# Returns 'undef' if nothing is handled
sub handleData
{
	my $Self = shift;
	my $FormRef = $Self -> FormRef();
	#my $FormRef = $Self -> VariableHash();
	#my %FormValues = %{ $FormRef };

	# Other magical vars
	my( $Statement, $Field, @Fields, @Values, @Queues, $RecordRef, $Ret );

	# Verify that we are attached to a table
	#if( $Self -> Table() eq "" )
	#{
	#	carp( "Form not bound to table" );
	#	return( undef );
	#}

	if( $FormRef -> { __xad_formname } ne $Self -> Name() )
	{
		return( undef );
	}

	# Check for delete.  Forms suck up any kind of delete
	if( $FormRef -> { __button } eq "DELETE" )
	{
	    # Prompt for confirmation
	    $Self -> _dmesg( "DELETE step 1", 3 );
	    $Self -> { SHOWMESSAGE } = $Self -> AskDelete();
	    return( undef );
	}

	if( $FormRef -> { action } eq "DELETE.FINAL" )
	{
	    if( $Self -> AllowDelete() )
	    {
		if( $FormRef -> { __button } eq "YES" )
		{
		    # Do the del
		    $Statement = "delete from " . $Self -> Table() .
			" where " . $Self -> PKey() . "=?";
		    
		    $Self -> _dmesg( "Deleting with statemement: " . $Statement, 3 );
		    $Self -> _dmesg( "Value for ? is: " . $FormRef -> { $Self -> VarPKey() }, 3 );
		    $Self -> { DATABASE } -> do( $Statement, undef, $FormRef -> { $Self -> VarPKey() } );
		    
		    # Maybe allow message later?
		    $Self -> { SHOWMESSAGE } = $Self -> OnDelete();
		    
		    return( 1 );
		}
		else
		{
		    $Self -> _Add( "Record not deleted<BR>\n" );
		    delete( $FormRef -> { action } );
		    return( 1 );
		}
	    }
	    else
	    {
		confess( "Illegal delete operation called" );
	    }
	}	
	
	# OK, if action is defined...
	if( exists( $FormRef -> { action } ) && ( $FormRef -> { __xad_formname } eq $Self -> Name() ) )
	{
	    # Propogate field values
	    $Self -> _FillFields( $FormRef );
	    if( $Self -> _ValidateFields() )
	    {
		# Grab preliminary data
		for $Field ( @{ $Self -> { FIELDS } } )
		{
			if( defined( $Field -> GenerateFieldForUpdateSQL() ) )
			{
			    push( @Fields, $Field -> GenerateFieldForUpdateSQL()  );

			    # Set the field value, then it can give us a
			    # proper SQL representation
			    $Field -> SetValue( $FormRef -> { $Field -> VarName() } );

			    push( @Values, $Field -> SqlValue() );
			    push( @Queues, "?" );

			    # Build the record ref for validation by the HOOK
			    # routines, if any
			    $RecordRef -> { $Fields[ -1 ] } = $Values[ -1 ];
			}
			else
			{
			    $Self -> _dmesg( "Skipping field '" . $Field -> Name . "'", 2 );
			}
		}

		# OK, update or add?
		if( $FormRef -> { action } eq "ADD" )
		{
			# Validate
			if( $Self -> HookAdd() )
			{
				$Ret = &{ $Self -> HookAdd() }( $RecordRef );
				if( $Ret )
				{
					$Self -> _Add( $Ret );
					return( undef );
				}
			}

			# Make a magi-add
			$Statement = "insert into " . $Self -> Table();
			$Statement .= " ( " . join( ", ", @Fields ) . " )";
			$Statement .= " values";
			$Statement .= " ( " . join( ", ", @Queues ) . " )";
		}
		elsif( $FormRef -> { action } eq "EDIT" )
		{
			# Validate
			if( $Self -> HookEdit() )
			{
				$Ret = &{ $Self -> HookEdit() }( $RecordRef );
				if( $Ret )
				{
					$Self -> _Add( $Ret );
					return( undef );
				}
			}

			# Create an update statement
			$Statement = "update " . $Self -> Table();
			$Statement .= " set ";

			$Statement .= join( ", ", map( $_ . "=?", @Fields ) );

			$Statement .= " where " . $Self -> PKey() . "=?";

			# Add our PKEY to the value listing
			push( @Values, $FormRef -> { $Self -> VarPKey() } );
		}
		else
		{
			confess( "Unknown action: " . $FormRef -> { ACTION } );
		}

		$Self -> _dmesg( "Current SQL is: " . $Statement, 2 );

		# Probably should exec here
		if( ! $Self -> { DATABASE } -> do( $Statement, undef, @Values ) )
		{
		    # Error!!
		    print "Content-type: text/plain\n\n";
		    print"An SQL error occurred while processing\n";
		    print $Statement . "\n";
		    print $Self -> { DATABASE } -> errstr . "\n";
		    confess( "I died" );
		}

		# Did something, so gotta return.  :)
		# Also pass our onexecgoto var up to our screen
		#if( $Self -> { SCREEN } )
		#{
		#    $Self -> { SCREEN } -> redirect( $Self -> OnExecGoto() );
		#}
		
		return( 1 );
	    }
	    else
	    {
		# Problem??
		$Self -> _dmesg( "Not all fields are valid", 3 );

		# Tell ourselves we are bad
		$Self -> { BADFORM } = 1;
		
		return( undef );
	    }
	}

	$Self -> _dmesg( "ACTION not set or not mine", 2 );
	$Self -> _dmesg( "ACTION is '" . $FormRef -> { action } . "'", 2 );
	$Self -> _dmesg( "NAME is '" . $FormRef -> { __xad_formname } . "'", 2 );
	$Self -> _dmesg( "My NAME is '" . $Self -> Name() . "'", 2 );

	return( undef );
}

# Variable attribs
sub addfield
{
	# Pushes a new field on the end of the form.  Returns the field
	# reference to be mangled.  A particular field can be grabbed
	# using the GetFieldByIndex or GetFieldByName methods.
	my $Self = shift;
	
	push( @{ $Self -> { FIELDS } }, Site::FieldSQL -> new( @_ ) );
	$Self -> { FIELDS } -> [ -1 ] -> SetDatabase( $Self -> { DATABASE } );
	$Self -> { FIELDS } -> [ -1 ] -> FormName( $Self -> Name() );
	
	return( $Self -> { FIELDS } -> [ -1 ] );
}

sub addfields
{
	my $Self = shift;
	# Takes an array of hash refs to pass to addfield
	for ( @_ )
	{
		$Self -> addfield( %{ $_ } );
	}
}

sub getnamedfieldCount
{
	my $Self = shift;

	# Get's the number of fields in me
	return( scalar( @{ $Self -> { FIELDS } } ) );
}

sub _ValidateFields
{
    # Asks each field to validate itself.  Returns 0 or 1
    my $Self = shift;

    my $OK = 1;
    
    for ( @{ $Self -> { FIELDS } } )
    {
	if( ! $_ -> IsValid() )
	{
	    # Dump error message
	    $Self -> { BADFORM } = 1;
	    $OK = 0;
	}	
    }

    return( $OK );
}

sub _DumpErrors
{
    my $Self = shift;

    # This should be called by the GenerateHTML part if
    # the BADFORM attribute is set
    for ( @{ $Self -> { FIELDS } } )
    {
	if( ! $_ -> IsValid() )
	{
	    # Dump error message
	    $Self -> _Add( $_ -> displayError() );
	}	
    }
}

sub _FillFields
{
    my $Self = shift;

    my %FormValues = %{ shift() };

    for ( @{ $Self -> { FIELDS } } )
    {
        if( $_ -> Type() ne "carry" )
	{
	    $_ -> Value( $FormValues{ $_ -> VarName() } );
	}
    }
}

sub _FieldDefaults
{
    my $Self = shift;

    $Self -> _dmesg( "Setting all field defaults", 3 );
    for ( @{ $Self -> { FIELDS } } )
    {
	if( $_ -> Type() ne "carry" )
	{
	    $Self -> _dmesg( "Calling default on: " . $_ -> Name, 3 );
	    $_ -> SetDefault();
	}
    }
}

sub registerScreen
{
	my $Self = shift;

	$Self -> { SCREEN } = shift;
}

sub DidUpdate
{
    my $Self = shift;

    return( $Self -> { DIDUPDATE } );
}

sub _Add
{
    my $Self = shift;
    print shift();
}

sub XAD_RUN
{

    my $Self = shift;

    print $Self -> { displayVALUE };
}

sub VarPKey
{
    my $Self = shift;

    return( $Self -> Name() . "." . $Self -> PKey() . "_pkey" );
}

sub DefaultDB
{
	shift();
	$Site::FormSQL::AUTOFIELDS{ DATABASE } = shift();
}

sub DefaultFormRef
{
	shift();
	$Site::FormSQL::AUTOFIELDS{ FORMREF } = shift();
}

# Gotta return true
1;
