package Site::Page;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use strict;
use vars qw( %AUTOFIELDS @ISA $OPTS );

use PObject;
@ISA = qw( PObject );

use CGI;
use Site::Template;
#require( "Site/cookie.lib" );

use DB_File;
use URI::Escape;
use HTML::Entities ();

%AUTOFIELDS = ( TITLE => "Untitled",
			SUBTITLE => undef,
			SESSION => undef,
			OPTIONS => undef,
			COOKIES => undef,
			USESESSIONS => 0,
			SESSIONID => undef,
			SESSIONFILE => '/tmp/default_session.%s.db',
			CONTENT => 'text/html',
			EXTRAHEADERS => undef,
			INITIALIZED => undef );

my( $q );

# Uses CGI now
$q = new CGI;
$OPTS = $q -> Vars();

sub import
{
	shift();
	my %opts;
	my( $l, $r );

	# Allow some options to change
	for ( @_ )
	{
		if( /=/ )
		{
			( $l, $r ) = split( /=/ );
			$opts{ $l } = $r;
		}else
		{
			$opts{ $_ } = 1;
		}
	}

	if( $opts{ usesessions } )
	{
		$AUTOFIELDS{ USESESSIONS } = 1;
	}

	if( $opts{ sessionfile } )
	{
		$AUTOFIELDS{ SESSIONFILE } = $opts{ sessionfile };
	}
	return(1);
}

sub OBJ_NEW
{
	my $self = shift;

	my %Session;
	my $SessionName;
	my $c;

	$self -> Cookies( [] );
	$self -> extraheaders( {} );

	if( $self -> UseSessions() )
	{
		if( $q -> cookie( 'sid' ) )
		{
			$SessionName = $self -> _GenerateSessionName( $q -> cookie( 'sid' ) );
			$self -> SessionID( $q -> cookie( 'sid' ) );
		}
		else
		{
			$self -> SessionID( $$ . time() );
 			$c = $q -> cookie( -name=>'sid',
						-path => '/',
                                  		-value=>$self -> SessionID() );
                                    		#-expires=>'+1h',
                                    		#-path=>'/cgi-bin/database',
                                    		#-domain=>'.capricorn.org',
                                    		#-secure=>1);
			push( @{ $self -> Cookies() }, $c );
			$SessionName = $self -> _GenerateSessionName( $$ . time() );
		}

		#$self -> _dmesg( "Tying to session: $SessionName", 3 );
		tie %Session, 'DB_File', $SessionName;

		# Put in some standard infos
		$Session{ lastaccessed } = time();

		$Session{ ip } = $main::ENV{ "REMOTE_ADDR" };

		$Session{ lasturi } = $Session{ thisuri };
		$Session{ thisuri } = $main::ENV{ "REQUEST_URI" };
		if( $Session{ lasturi } ne $Session{ thisuri } )
		{
			$Session{ lastdistincturi } = $Session{ lasturi };
		}

		$self -> { SESSION } = \%Session;
	}

	# Save all the hashes
	$self -> { OPTIONS } = $OPTS;

	return( $self );
}

sub header
{
	my $self = shift;
	print $q -> header( -content_type => $self -> content(),
				-cookies => $self -> cookies(),
				%{ $self -> extraheaders() } );
}

sub start
{
	my $self = shift;
	$self -> header();
	$self -> _start();
}

sub _start
{
	my $Self = shift;

 	my $t = Site::Template -> new();
	if( -f 'header.tmpl' )
	{
		$t -> load( 'header.tmpl' );
	}else
	{
		$t -> template( $Self -> _DEFAULT_HEADER() );
	}

	$t -> registerdefault( $Self );
	$t -> display();
}

sub _DEFAULT_HEADER
{ 
    return( qq{
<HTML>
  <HEAD>
    <TITLE><t:var name=title/> - <t:var name=subtitle/></TITLE>
  </HEAD>
  
  <BODY>
} );
}


sub end
{
	my $self = shift;
	$self -> _end();
}

sub _end
{
	my $Self = shift;
 	my $t = Site::Template -> new();
	if( -f 'footer.tmpl' )
	{
		$t -> load( 'footer.tmpl' );
	}else
	{
		$t -> template( $Self -> _DEFAULT_FOOTER() );
	}

	$t -> registerdefault(  $Self );
	$t -> display();
}

sub _DEFAULT_FOOTER
{
    return( qq{
    </BODY>

</HTML>
} );
}

sub set_cookie
{
	my $Self = shift;
	my %Hash = @_;
	my $c;

	for( keys( %Hash ) )
	{
                        $c = $q -> cookie( -name=>$_,
                                                -value=>$Hash{$_} );
                                                #-expires=>'+1h',
                                                #-path=>'/cgi-bin/database',
                                                #-domain=>'.capricorn.org',
                                                #-secure=>1);
                        print $q -> header( -cookie => $c );
	}
}

sub redirect
{
	my $Self = shift;
	print $q -> redirect( shift() );
	exit( 0 );
}

sub _GenerateSessionName
{
	my $Self = shift;

	my $SessionID = shift;

	return( sprintf( $Self -> SessionFile(), $SessionID ) );
}

sub escape_uri
{
	my $Self = shift;
	my $Text = uri_escape( shift() );
	# Doesn't escape out &?
	$Text =~ s/&/\%26/g;
	return( $Text );
}

sub encode_entities
{
	my $Self = shift;

	return( HTML::Entities::encode( shift() ) );
}

sub decode_entities
{
	my $Self = shift;
	return( HTML::Entities::decode( shift() ) );
}

# All these are legacy support functions
#sub Initialize { return( shift() -> initialize( @_ ) ) }

1;
 

__END__

=head1 NAME

Site::Page - Basic site page handling

=head1 SYNOPSIS

  use Site::Page;
  $p = Site::Page -> new();
  $p -> title( 'My Page' );
  $p -> start();

 ...

  $p -> end();

=head1 DESCRIPTION

Blah

=head1 AUTHOR

Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.

This program is free software  and is provided "as is" without
express or implied warranty. You can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1).

=cut
