package Site::SQLFieldBase;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use strict;
use vars qw( @ISA );

use PObject;
@ISA = qw( PObject );

use Site::Page;

%Site::SQLFieldBase::AUTOFIELDS = (

# This contributes to the html field name
			NAME => undef,
# SQL field this is bound to
			BOUNDTO => undef,
# Label on the HTML form to be used
			LABEL => undef,

# Field type
#  text
			TYPE => undef,

# Enabled means the field can be edited in the browser
			ENABLED => 1,

# Locked means the field will not be part of the update SQL,
# it's still present in the select part.
			LOCKED => 0,

# Visible controls if the field is visible on the browser
			VISIBLE => 1,

# If the field is not bound to an SQL field, this should be set
			VALUE => undef,
# Default value for new records.  Overrides value in case of conflict
			DEFAULT => undef,

# Set by the container object, the container should provide an
# _HTMLNAME function for creating the HTML names
			CONTAINER => undef,

# If set to false, then it will not be autoescaped for display
			AUTOESCAPE => 1,

# Sets up for auto-pre/post pend.  Needs to be handled by the container
			HREF => undef
		);



# For the sql field names, the user class should weed out undef
# vars because they are not relevant to the task at hand.
sub SQLValue
{
	my $Self = shift;
	return( $Self -> Value() );
}

sub SQLSelect
{
	my $Self = shift;

	return( $Self -> BoundTo() );
}

sub SQLUpdate
{
	my $Self = shift;

	# If enabled and not locked....
	if( ! $Self -> Locked() ) 
	{
		return( $Self -> BoundTo() );
	}

	return( undef );
}

# This really should be overriden by the final class modules
sub _HTMLField
{
	my $Self = shift;
	return( "Class did not override field html" );
}

sub _HTMLLabel
{
	my $Self = shift;

	if( $Self -> Label() )
	{
		return( $Self -> Label() );
	}elsif( $Self -> BoundTo() )
	{
		return( $Self -> BoundTo() );
	}elsif( $Self -> Name() )
	{
		return( $Self -> Name() );
	}
}

sub SetDefault
{
	my $Self = shift;

	$Self -> { VALUE } = $Self -> Default();
}

# These are the ACTUAL methods to call by any user classes
sub HTMLField
{
	my $Self = shift;

	my $RVal;

	if( $Self -> Visible() )
	{
		if( $Self -> Enabled() )
		{
			$RVal = $Self -> _HTMLField();
		}else
		{
			# Not enabled, no editing, just display it
			$RVal = $Self -> _HTMLDisplayValue();
		}
	}

	# If I'm not locked I must include something
	if( ( ( ! $Self -> Locked() ) &&
		( $Self -> Visible() ) &&
		( ! $Self -> Enabled() ) ) ||
		( ! $Self -> Visible() ) )
	{
		# Just a hidden me
		$RVal .= '<INPUT TYPE=HIDDEN NAME=' . 
			$Self -> _HTMLNAME() .
			' VALUE="' .
			$Self -> _HTMLValue() .
			'">';
	}

	return( $RVal );
}

sub HTMLLabel
{
	my $Self = shift;
	if( $Self -> Visible() )
	{
		return( $Self -> _HTMLLabel() );
	}
	return( undef );
}

sub _HTMLValue
{
	# Returns the value HTML'ized
	my $Self = shift;
	return( Site::Page -> encode_entities( $Self -> Value() ) );
}

# If the value shown is different than the value used,
# then this should be overriden in the subclass
sub _HTMLDisplayValue
{
	my $Self = shift;
	if( $Self -> AutoEscape() )
	{
		return( $Self -> _HTMLValue() );
	}else
	{
		return( $Self -> Value() );
	}
}

sub HTMLDisplay
{
	my $self = shift;
	if( $self -> visible() )
	{
		return( $self -> _HTMLDisplayValue() );
	}else
	{
		return( '' );
	}
}

sub _HTMLNAME
{
	# Returns the fully qualified field name
	my $Self = shift;
	if( $Self -> Container() )
	{
		return( $Self -> Container() -> _HTMLNAME() .
			".fields." .
			$Self -> Name() );
	}
	return( $Self -> Name() );
}

sub htmlname
{
	return( shift() -> _HTMLNAME() );
}

# This will handle setting my name/binding in the container object,
# if we're contained
sub OBJ_HOOK_NAME
{
	my $Self = shift;
	my $NewName = shift;
	my $OldName = $Self -> { NAME };
	my $Fields;

	if( $Self -> Container() )
	{
		$Fields = $Self -> Container() -> NamedFields();
		# Delete our old name
		if( $OldName !~ /^\s*$/ )
		{
			delete( ${ $Fields }{ $OldName } );
		}

		if( $NewName !~ /^\s*$/ )
		{
			#print "Nameing: $NewName\n";
			$Fields -> { $NewName } = $Self;
		}
	}

	return( $NewName );
}

# To delete this field
sub remove
{
	my $Self = shift;
	$Self -> Container() -> _DELETEFIELD( $Self );
}

sub OBJ_HOOK_BOUNDTO
{
	my $Self = shift;
	my $NewName = shift;
	my $OldName = $Self -> { BOUNDTO };
	my $Fields;

	if( $Self -> Container() )
	{
		$Fields = $Self -> Container() -> BoundFields();
		if( $NewName !~ /^\s*$/ )
		{
			$Fields -> { $NewName } = $Self;
		}

		# Delete our old name
		if( $OldName !~ /^\s*$/ )
		{
			delete( ${ $Fields }{ $OldName } );
		}
	}

	if( ! $Self -> Name() )
	{
		# Auto-set name too
		$Self -> Name( $NewName );
	}

	return( $NewName );
}

sub SetValue
{
	my $Self = shift;

	$Self -> { VALUE } = shift();
}

sub disabled
{
	# Shorthand for locked=1/enabled=0
	my $Self = shift;
	if( shift() )
	{
		$Self -> Locked( 1 );
		$Self -> Enabled( 0 );
	}else
	{
		$Self -> Locked( 0 );
		$Self -> Enabled( 1 );
	}
}

sub OBJ_HOOK_HREF
{
	# If href is set, that means we need to not auto-escape
	my $Self = shift;
	my $Val = shift;

	$Self -> AutoEscape( 0 );
	return( $Val );
}

1;
