package Site::SQLForm;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use strict;
use vars qw( @ISA %AUTOFIELDS $AUTOSETTINGS );

use Site::SQLContainer;
@ISA = qw( Site::SQLContainer );

use Site::Page;
use Site::Template;
use DB_File;

$AUTOSETTINGS = 1;
%AUTOFIELDS = (
			FORMACTION => $::ENV{ SCRIPT_NAME },
			TEMPLATE => undef,
			TEMPLATEFILE => undef,
			DELETETEMPLATE => undef,

			STATIC_FORM => 0,

# Currently they just remove the buttons...
			ALLOWADD => 1,
			ALLOWEDIT => 1,
			ALLOWDELETE => 1,
			DELETEBUTTON => 'Delete',
			ADDBUTTON => 'Add',
			EDITBUTTON => 'Edit',
			DELETEYESBUTTON => 'Yes',
			DELETENOBUTTON => 'No',
			MESSAGE => undef,
			PKEYVALS => undef,
			USEDATA => undef,
			LASTACTION => undef,
			TXFILE => undef,
			TXREF => undef,
# Set to 1 then shows errors in web page
			ERRORS => 0,
			NOTFOUND => "Row not found",
			BADFORM => 0,

# Set to 1 then doesn't got edit -> add mode, stays in edit mode
# probably needs a better name
			ALWAYSEDIT => 0
	);

sub handle
{
	# Return true if handled something, false if not
	# Will also set BADFORM if I'm bad.  :)
	my $self = shift;

	# Check for PKEY values and fill them in if applicable
	$self -> _FillPKeys();
	#$self -> _MapBoundFields();

	my $Do = $self -> _WhatToDo();
	my( $MagicQ, $TXNO );
	my @FieldNames = $self -> _MODIFYFIELDS();
	my @Values; # = $self -> _Values();

	#print "$Do<BR>";

	# Gather fields values
	#for( keys( %{ $self -> Opts() } ) )
	#{
	#	print $_ . ": " . $self -> Opts() -> { $_ } . "<BR>";
	#}

	# If badform is already set, then outside auth has taken over
	if( $self -> badform() )
	{
		return( undef );
	}

	# Do?
	if( ! grep( $Do eq $_, 'UPDATE',
				'ADD',
				'DELETE' ) )
	{
		return( undef );
	}

	# If no dbh, assume you're using it for a form with defaults
	if( $self -> static_form() )
	{
		$self -> lastaction( 'NOTHING' );
		return( 1 );
	}

	for( @FieldNames )
	{
		#print $self -> getnamedfield( $_ ) -> _HTMLNAME();
		# Set the value in the field
		$self -> getboundfield( $_ ) -> SetValue( $self -> Opts() -> 
			{ $self -> getboundfield( $_ ) -> _HTMLNAME() } );

		push( @Values, $self -> getboundfield( $_ ) -> SQLValue );
	}

	if( $Do eq "UPDATE" )
	{
		# Is allowed?
		if( ! $self -> allowedit() )
		{
			return( undef );
		}

		# Perform an update query
		$MagicQ = 'update ' .
				$self -> Table() .
				' set ' .
				join( ", ", map( $_ . "=?",
					$self -> _MODIFYFIELDS() ) ) .
				' where ' .
				join( " AND ", map( $_ . "=?",
					@{ $self -> PKeys() } ) );

		# Add PKEYS to the values array
		for( @{ $self -> PKeys } )
		{
			push( @Values, $self -> Opts() -> {
					$self -> _HTMLNAME() . '.pkeys.' . $_ }
				);
		}

		$self -> LastAction( "UPDATE" );
	}elsif( $Do eq "ADD" )
	{
		# Is allowed?
		if( ! $self -> allowadd() )
		{
			return( undef );
		}

		# Perform an insert query
		$MagicQ = 'insert into ' .
			$self -> Table() .
			' ( ' .
			join( ", ", $self -> _MODIFYFIELDS() ) .
			' ) values ( ' .
			join( ", ", map( '?', $self -> _MODIFYFIELDS() ) ) .
			' ) ';

		$self -> LastAction( "ADD" );
	}elsif( $Do eq "DELETE" )
	{
		# Is allowed?
		if( ! $self -> allowdelete() )
		{
			return( undef );
		}

		# Perform deleteooo
		$MagicQ = 'delete from ' .
				$self -> Table() . 
				' where ' .
				join( " AND ", map( $_ . "=?",
						@{ $self -> PKeys() } ) );
		@Values = ();
		for( @{ $self -> PKeys } )
		{
			push( @Values, $self -> Opts() -> {
					$self -> _HTMLNAME() . '.pkeys.' . $_ }
				);
		}
		$self -> LastAction( "DELETE" );
	}

	#print $MagicQ;
	#print "(" . join( ", ", @Values ) . ")";

	# Do it!
	my $Status;

	# Check TX support, then check if TXOK
	if( $self -> txref() )
	{
		# Check the transaction
		$TXNO = $self -> Opts() -> { $self -> _TXPROTECT() };
		if( ! exists( $self -> txref() -> { $TXNO } ) )
		{
			# Doesn't exist, OK!
			$Status = $self -> DBH() -> do( $MagicQ, undef, @Values );
			$self -> txref() -> { $TXNO } = 1;
		}else
		{
			#print STDERR "TX Duplicate\n";
			$Status = 1;
		}
			
	}else
	{
		$Status = $self -> DBH() -> do( $MagicQ, undef, @Values );
	}

	if( ! defined( $Status ) )
	{
		# Baddy!
		$self -> { BADFORM } = 1;
		$self -> AddMessage( "<PRE>An SQL error occurred: '" .
				$self -> DBH() -> errstr() . "'</PRE>" );
		$Do = 0;
		$self -> lastaction( 'ERROR' );

		# Delete transaction
		if( $self -> txref() )
		{
			# TXNO set above
			delete( $self -> txref() -> { $TXNO } );
		}
	}

	return( $Do );
}

sub display
{
	my $self = shift;

	# Decide what to do based on vars
	if( ! $self -> _FillFields() )
	{
		# Error!
		return( undef );
	}

	#print $self -> _WhatForm();

	# Once fields are filled, perform field to href bindings
	$self -> { USEDATA } = {
			begin => $self -> HTMLBegin(),
			end => $self -> HTMLEnd(),

			message => $self -> Message(),

			add => $self -> HTMLAdd(),
			edit => $self -> HTMLEdit(),
			delete => $self -> HTMLDelete(),

			yes => $self -> HTMLDeleteYes(),
			no => $self -> HTMLDeleteNo(),

			fields => []
		};
	for( @{ $self -> ListFields() } )
	{
		$self -> _AddFieldToHREF( $self -> UseData(), $_ );
	}

	if( $self -> _WhatForm() eq "DELETE" )
	{
		$self -> _DisplayDelete();
	}else
	{
		$self -> _DisplayForm();
	}
}

sub _WhatToDo
{
	# Look at the opts and decide what to do
	my $self = shift;
	my $ActVal = $self -> Opts() -> { $self -> _ACTION() };
	my $ButVal = $self -> Opts() -> { $self -> _Button() };

	#print $ActVal . "::" . $ButVal . "<BR>";
	if( $ButVal eq $self -> DeleteButton() )
	{
		# If they pushed delete then ask if they are sure
		return( "ASKDELETE" );
	}elsif( $ActVal eq "DELETE" )
	{
		# They said yes?
		if( $ButVal eq $self -> DeleteYesButton() )
		{
			return( "DELETE" );
		}else
		{
			# Nope
			return( "NODELETE" )
		}
	}elsif( $ActVal eq "UPDATE" )
	{
		return( "UPDATE" );
	}elsif( $ActVal eq "ADD" )
	{
		return( "ADD" );
	}elsif( $self -> PKeyVals() )
	{
		return( "ASKUPDATE" );
	}else
	{
		# Nothing means askadd
		return( "ASKADD" );
	}
}

sub _WhatForm
{
	# Just a logic filter for the above
	my $self = shift;
	my $Do = $self -> _WhatToDo();

	# This will tell what form to display and the method it should do
	if( grep( $_ eq $Do, 'ASKADD',
				'ADD',
				'UPDATE',
				'DELETE',
				'NODELETE' ) )
	{
		if( $self -> alwaysedit() )
		{
			return( "UPDATE" );
		}else
		{
			return( "ADD" );
		}
	}elsif( $Do eq 'ASKUPDATE' )
	{
		return( "UPDATE" );
	}elsif( $Do eq "ASKDELETE" )
	{
		# Munge the PKEYS out of the query
		$self -> PKeyVals( [] );
		for( @{ $self -> PKeys } )
		{
			push( @{ $self -> PKeyVals() }, $self -> Opts() -> {
					$self -> _HTMLNAME() . '.pkeys.' . $_ }
				);
		}
		return( "DELETE" );
	}

	return( undef );
				
}

sub _FillFields
{
	# Call proper filler based on action to take
	my $self = shift;
	my $Do = $self -> _WhatForm();

	# Bad form?
	if( $self -> badform() )
	{
		return( $self -> _FillFieldsBad() );
	}

	# If askadd or just updated or just deleted, then fill defaults
	if( $Do eq 'ADD' )
	{
		return( $self -> _FillFieldsAdd() );
	}elsif( $Do eq "UPDATE" )
	{
		return( $self -> _FillFieldsUpdate() );
	}elsif( $Do eq "DELETE" )
	{
		return( $self -> _FillFieldsUpdate() );
	}
}

sub _FillFieldsUpdate
{
	# This will get values from an SQL query
	my $self = shift();

	my @PKeys = @{ $self -> PKeys() };
	my @Fields = $self -> _SELECTFIELDS();
	my @Values;

	# Generate the select query
	my $MagicQ = 'select ' . join( ", ", @Fields ) .
		' from ' . $self -> Table() .
		' where ' .
		join( " AND ", map( $_ . "=?", @PKeys ) );

	my $sth = $self -> DBH() -> prepare( $MagicQ );
	$sth -> execute( @{ $self -> PKeyVals() } );

	my $Ref = $sth -> fetchrow_hashref();

	if( ! defined( $Ref ) )
	{
		$self -> AddMessage( "Row not found" );

		# Should break it down a bunch here
		if( $self -> errors() )
		{
			print $self -> notfound();
		}
		return( undef );
	}

	for( keys( %{ $Ref } ) )
	{
		$self -> getboundfield( $_ ) -> SetValue( $Ref -> { $_ } );
	}

	return( 1 );
}

sub _FillFieldsAdd
{
	my $self = shift;

	for( @{ $self -> HTMLFields() } )
	{
		$_ -> SetDefault();
	}

	return( 1 );
}

sub HTMLFields
{
	my $self = shift;
	return( $self -> ListFields() );
}

sub _FillFieldsBad
{
	# This will get values from the environment
	my $self = shift;
	for( @{ $self -> HTMLFields() } )
	{
		$_ -> Value( $self -> Opts() -> { $_ -> htmlname() } );
	}
	return( 1 );
}

sub _DisplayForm
{
	my $self = shift;

	my $Ret;

	# Performs the actual form display
	if( ! $self -> Template() )
	{
		$self -> Template( Site::Template -> new() );

		if( $self -> templatefile() )
		{
			$self -> template() -> load( $self -> templatefile() );
		}else
		{
			$self -> Template() -> Template( $self -> _DefaultTemplate() );
		}
	}

	$self -> Template() -> registerdefault( $self -> UseData() );
	$self -> Template() -> display();
}

sub _DisplayDelete
{
	my $self = shift;
	if( ! $self -> DeleteTemplate() )
	{
		$self -> DeleteTemplate( Site::Template -> new() );
		$self -> DeleteTemplate() ->
			Template( $self -> _DefaultDeleteTemplate() );
	}

	$self -> DeleteTemplate() -> registerdefault( $self -> UseData() );
	$self -> DeleteTemplate() -> display();
}

sub _DefaultTemplate
{
	# Return a default template
	return( qq{<t:var name=begin/>
<t:var name=message/>
<TABLE BORDER=0>
<t:loop name=fields>
<TR>
<TD><t:iftrue name=label><B><t:var name=label/>:</B></t:iftrue></TD>
<TD><t:var name=field/></TD>
</TR>
</t:loop>
</TABLE>
<t:var name=add/>
<t:var name=edit/>
<t:var name=delete/>
<t:var name=end/>
} );

}

sub _DefaultDeleteTemplate
{
	return( qq{<t:var name=begin/>
Delete This record?<BR>

<TABLE BORDER=0>
<t:loop name=fields>
<TR>
<TD><t:iftrue name=label><B><t:var name=label/>:</B></t:iftrue></TD>
<TD><t:var name=display/></TD>
</TR>
</t:loop>
</TABLE>

<t:var name=yes/>
<t:var name=no/>
<t:var name=end/>
} );
}

sub HTMLBegin
{
	my $self = shift;
	my $Action = $self -> FormAction();
	my $Do = $self -> _WhatForm();

	my $Ret = "<FORM ACTION=$Action METHOD=GET>" .
		'<INPUT TYPE=HIDDEN NAME=' . 
		$self -> _ACTION() .
		' VALUE="' . 
		$Do . 
		'"><INPUT TYPE=HIDDEN NAME=' .
		$self -> _TXPROTECT() .
		' VALUE="' .
		time() . '.' . $$ .
		'">';

	if( $Do ne "ADD" )
	{
		# Add pkey vals
		my @Vals = @{ $self -> PKeyVals() };
		for( @{ $self -> PKeys() } )
		{
			$Ret .= '<INPUT TYPE=HIDDEN NAME=' .
				$self -> _HTMLNAME() .
				'.pkeys.' .
				$_ .
				' VALUE="' .
				Site::Page ->
					encode_entities( shift( @Vals ) ) .
				'">';
		}
	}

	# Spew forth the carrables!
	for ( keys( %{ $self -> Carry() } ) )
	{
		$Ret .= '<INPUT TYPE=HIDDEN NAME=' .
			$_ .
			' VALUE="' .
			Site::Page ->
				encode_entities( $self -> Carry() -> { $_ } ) .
			'">';
	}

	return( $Ret );
}


sub HTMLEnd
{
	return( qq{</FORM>} );
}

sub HTMLLabel
{
	my $self = shift;
	my $Fname = shift;

	return( $self -> getnamedfield( $Fname ) -> HTMLLabel() );
}

sub HTMLField
{
	my $self = shift;
	my $Fname = shift;

	return( $self -> getnamedfield( $Fname ) -> HTMLField() );
}

sub HTMLAdd
{
	# The add button, if applicable
	my $self = shift;

	if( ( $self -> _WhatForm() eq "ADD" ) &&
		( $self -> allowadd() ) )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$self -> _Button() .
			' VALUE="' . 
			$self -> AddButton() .
			'">' );
	}
	return( undef );
}

sub HTMLEdit
{
	# The edit button
	my $self = shift;
	if( ( $self -> _WhatForm() eq "UPDATE" ) &&
		( $self -> allowedit() ) )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$self -> _Button() .
			' VALUE="' . 
			$self -> EditButton() .
			'">' );
	}

	return( undef );
}

sub HTMLDelete
{
	# The delete button
	# Only on the update form
	my $self = shift;

	if( ( $self -> _WhatForm() eq "UPDATE" ) &&
		( $self -> allowdelete() ) )
	{
		return( '<INPUT TYPE=SUBMIT NAME=' . 
			$self -> _Button . 
			' VALUE="' . $self -> DeleteButton() . '">' ); 
	}
	return( undef );
}

sub HTMLDeleteYes
{
	my $self = shift();

	return( '<INPUT TYPE=SUBMIT NAME=' . 
			$self -> _Button . 
			' VALUE="' . $self -> DeleteYesButton() . '">' ); 
}

sub HTMLDeleteNo
{
	my $self = shift();

	return( '<INPUT TYPE=SUBMIT NAME=' . 
			$self -> _Button . 
			' VALUE="' . $self -> DeleteNoButton() . '">' ); 
}

sub HTMLDisplay
{
	my $self = shift;
	return( $self -> getnamedfield( shift() ) -> displayValue() );
}

sub _Button
{
	my $self = shift;

	return( $self -> _HTMLNAME() . ".__.button" );
}

sub AddMessage
{
	my $self = shift;
	$self -> { MESSAGE } .= shift() . "\n";
}

sub _FillPKeys
{
	my $self = shift;
	my @Arr;
	my $IsSet = 0;

	for( @{ $self -> _HTMLPKeys() } )
	{
		push( @Arr, $self -> Opts() -> { $_ } );
		if( defined( $Arr[ -1 ] ) )
		{
			$IsSet = 1;
		}else
		{
			# Flip back to zero if ANY are not defined
			$IsSet = 0;
		}
	}

	if( $IsSet )
	{
		$self -> { PKEYVALS } = \@Arr;
	}
}

sub _HTMLPKeys
{
	# This will be configurable later, for now the pkeys are not fully
	# qualified
	my $self = shift;

	return( $self -> PKeys() );
}

sub _AddFieldToHREF
{
        my $self = shift;
        my $href = shift;
        my $Field = shift;

        my( $useref, $fieldref );
  
        $useref = {
                        field => $Field -> HTMLField(),
                        label => $Field -> HTMLLabel(),
                        display => $Field -> HTMLDisplay(),
                };

        push( @{ $href -> { fields } }, $useref );
        $href -> { byname } -> { $Field -> name() } = $useref;
        $href -> { byboundto } -> { $Field -> boundto() } = $useref;
}

sub transaction_support
{
	# Uses a db file, takes an optional file name
	shift();
	my %tx;
	my $File = shift() || 'SQLForm.tx.db';
	$AUTOFIELDS{ TXFILE } = $File;

	# Now tie the TXREF
	tie %tx, 'DB_File', $File;
	$AUTOFIELDS{ TXREF } = \%tx;
}

# A shortcut function
sub htmlname
{
	return( shift() -> getnamedfield( shift() ) -> htmlname() );
}

sub htmlboundname
{
	return( shift() -> getboundfield( shift() ) -> htmlname() );
}

sub DUMP_TEMPLATE
{
	# For making custom templates.  This dumps a template for the
	# current data
	my $self = shift();
	my( $ret, $name );

	if( ! ref( $self ) )
	{
		return( $self -> _DefaultTemplate() );
	}

	$ret = q{
<t:var name=begin/>
<t:var name=message/>
<TABLE BORDER=0>
<t:with name=byname>
};

	for( @{ $self -> listfields() } )
	{
		$name = $_ -> name();
		$ret .= qq{
<TR>
<TD><B><t:var name=$name:label/>:</B></TD>
<TD><t:var name=$name:field/></TD>
</TR>
};
	}

	$ret .= q{
</TABLE>
</t:with>
<t:var name=add/>
<t:var name=edit/>
<t:var name=delete/>
<t:var name=end/>
};

	return( $ret );
}

sub get_named_values
{
	# Grabs form values from $opts and returns them as a hash
	my $self = shift;
	my %ret = ();
	my $tmp;

	for( @{ $self -> listfields() } )
	{
		$tmp = $_ -> name();
		$ret{ $tmp } = $self -> opts() -> { $_ -> htmlname() };
	}

	return( \%ret );
}

1;

__END__

=head1 NAME

Site::SQLForm - HTML forms bound to SQL tables

=head1 SYNOPSIS

  use Site::SQLForm;

  $f = Site::SQLForm -> new();
  $f -> dbh( $dbh );

  $f -> table( 'test' );
  $f -> addfields( { ... } );

  if( $f -> handle() )
  {
    print "Record handled!";
  }

  $f -> display();

=head1 DESCRIPTION

SQLForm inherits all field handling methods from Site::SQLContainer.
Read the pod for that modules for the basic information about how
to get the class to actually "do" anything.

=head2 handle

This handles any incoming requests, add, edit or delete.

=head2 display

This actually displays the form code

=head2 allow(add/edit/delete)

Enables and disables adding, editing, and deleting records
from the database.  Default is '1' for all of them.

=head2 (add/edit/delete/deleteno/deleteyes)button

The text on the front of the buttons.

=head2 transaction_support

If set to a filename, it will eliminate duplicate transactions by assigning
a txid to each transaction and only performing it once.  Uses the DB_File
to keep track of txid's.

=head2 lastaction

After a succesful handle() call, this will be either "ADD", "EDIT", or
"DELETE" telling what the form actually did.

=head2 deletetemplate

Template for deleting a record, for a custom delete form.  Adds a
'yes' and 'no' variable for the t:var thing for the yes and no
buttons.

=head2 badform

You can check some form values and set this to '1' if things aren't right.
It will force the values back in to the fields and make the user submit
them again, hopefully after changing them.

=head1 Templates

(if you just want quick templates, see the DUMP_TEMPLATE method)

You can use templates to customize the look of forms.  The default
look is to have labels down the left with fields on the right in
a table, like:

 label: field
 label: field
 ...

If you wanted though, you could change it to be:

 label: field
 label: field field field
        label: field <submit>

Or anything else using templates.

The default template looks like this:

<t:var name=begin/>
<t:var name=message/>
<TABLE BORDER=0>
<t:loop name=fields>
<TR>
<TD><t:iftrue name=label><B><t:var name=label/>:</B></t:iftrue></TD>
<TD><t:var name=field/></TD>
</TR>
</t:loop>
</TABLE>
<t:var name=add/>
<t:var name=edit/>
<t:var name=delete/>
<t:var name=end/>

You can reference the fields in the fields[] loop one by one in the added()
order, or you can do things byname/boundto (which is more likely for custom
forms) like this:

<t:var name=begin/>
<t:var name=message/>

<t:with name=byname>
<t:var name=name:label/>: <t:var name=name:field/>
</t:with>

<t:var name=add/>
<t:var name=edit/>
<t:var name=delete/>
<t:var name=end/>

There are several values defined for use, with begin/end being required.

=head2 var begin

This contains all the addcarries, any hidden stuff needed, and the form
declaration.  This is required.

=head2 var message

Error messages for the form will be here.  If you don't include it, errors
will not be displayed to the browser!

=head2 var byname

Under the byname variable are all fields by name, with each field having
a :label, :field, and :display for the display value.

=head2 var byboundto

Same as byname except by bound to fields.

=head2 var fields

A loop[] variable, and for each item has a label, field and display
similar to the byname variable.

=head2 var add/edit/delete

These are the add/edit/delete BUTTONS.  Include or exclude them as
you wish.

=head3 var end

This is required and closes up the form.

=head1 AUTHOR

Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.

This program is free software  and is provided "as is" without
express or implied warranty. You can redistribute it and/or
modify it under the same terms as Perl itself.

=head1 SEE ALSO

perl(1).

=cut
