package Site::Template;
# LICENSE
#
# Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.
#
# This program is free software  and is provided "as is" without
# express or implied warranty. You can redistribute it and/or
# modify it under the same terms as Perl itself.
#

use strict;
use vars qw( @ISA %AUTOFIELDS %READFIELDS $AUTOSETTINGS @Libs );

use PObject;
@ISA = qw( PObject );

use Cwd qw( abs_path );
use Site::Page;
use HTML::Entities;
use Site::Template::Element;
use Site::Template::DocTree;

use Data::Dumper;

$AUTOSETTINGS = 1;

# This allows templates based on objects.  Crush means kill all spaces
%AUTOFIELDS = ( TEMPLATE => undef,
			COMMENTCHAR => undef,
			CACHED => undef,
			SAVECACHE => undef,
			CACHEDIR => 'cache',
			CRUSH => undef,
			PRINT => 1,
			# If print is false, the output will
			# be generated in to OUTPUT
			OUTPUT => undef,

			# If decode entities is true, then HTML
			# entites will be decoded during output
			# this can be used for making other
			# kinds of templates that may require
			# < or > or & that use a plain text input,
			# such as latex or C
			DECODEENTITIES => 0,

			# This should be a function reference.
			# it should take 1 argument ( the data )
			# and return 1 string
			CUSTOM_ESCAPE => undef ); 

%READFIELDS = ( 
				OBJECTS => undef );

@Libs = ( '.', 'Site' );
			
sub OBJ_NEW
{
	my $self = shift;

	$self -> { OBJECTS } = {};

	return( $self );
}

sub load
{
	my $self = shift;

	my( $SaveSep, $CacheFile );
	my( $CacheAge, $FileAge );
	my $FileName = shift();

	local *IN;

	$self -> { TEMPLATE } = "";
	$self -> { CACHED } = "";

	if( $FileName !~ m#^/# )
	{
		$FileName = $self -> resolvefile( $FileName );
	}

	# Check if a cached version exists
	$CacheFile = $FileName;
	$CacheFile =~ s#_#__#g;
	$CacheFile =~ s#/#_#g;
	$CacheFile = $self -> CacheDir() . "/" . $CacheFile;
	if( -f $CacheFile )
	{
		# Check if it's up-to-date (by last-m-time)
		$CacheAge = ( stat( $CacheFile ) )[ 9 ];
		$FileAge = ( stat( $FileName ) )[ 9 ];

		# If cache is greater(newer) than file
		if( $CacheAge > $FileAge )
		{
			$self -> Cached( $CacheFile );
			return( 0 );
		}
	}
	
	open( IN, $FileName ) || $self -> _dfatal( "Could not open template: '" . $FileName . "'" );

	$SaveSep = $/;
	undef $/;

	$self -> { TEMPLATE } = <IN>;

	$/ = $SaveSep;

	close( IN );

	# Tells it to write out a cache file later
	if( -d $self -> CacheDir() )
	{
		$self -> SaveCache( $CacheFile );
	}
}

sub register
{
	my $self = shift;
	my $Obj;
	my $Name;

	while( @_ )
	{
		$Name = shift();
		$Obj = shift();

		$self -> Objects() -> { $Name } = $Obj;
	}
}

sub registerdefault
{
	my $self = shift;

	# Takes a hashref or obj, makes it the objects hashref
	$self -> { OBJECTS } = shift;
}

sub display
{
	my $self = shift;

	$self -> { OUTPUT } = undef;
	if( $self -> Cached() )
	{
		$self -> _CACHEDISPLAY();
	}
	else
	{
		$self -> _PARSEDISPLAY();
	}

	return( $self -> Output() );
}

sub _CACHEDISPLAY
{
	my $self = shift;

	my( $VAR1, $SaveSep, $File );
	local *IN;

	open( IN, $self -> Cached() );

	$SaveSep = $/;
	undef $/;
	$File = <IN>;
	$/ = $SaveSep;

	close( IN );

	eval( $File );

	$self -> _PROCESSTREE( $VAR1, $self -> Objects() );
}

sub _PARSEDISPLAY
{
	my $self = shift;

	my $OutPut = 't:nul />' . $self -> Template();

	my( $Tag, $TopLevel, $ThisNode, $Data, $com_char );
	my( $ShouldPush, $ShouldPop, $Rest, $LastTag, $CrushNL );

	local *OUT;

	$TopLevel = Site::Template::DocTree -> new();
	$ThisNode = $TopLevel;

	$ThisNode -> Data( Site::Template::Element -> new() );
	$ThisNode -> Data() -> Tag( 'nul' );

	# Clean the pre-parse file of comments...
	$com_char = $self -> commentchar();
	if( $com_char )
	{
		$OutPut =~ s/$com_char.*?\r?\n//gs;
	}

	$CrushNL = 0;
	for ( split( /</, $OutPut ) )
	{
		# Check if this is a template tag
		if( $_ =~ m#^/?t:.*>#i )
		{
			# A tag, set shouldpush/pop flagging
			$ShouldPush = 0;
			$ShouldPop = 0;
			if( $_ =~ m#^/# )
			{
				s#^/##;
				$ShouldPop = 1;
			}
			else
			{
				$ShouldPush = 1;
			}

			if( $_ =~ m#/># )
			{
				$ShouldPop = 1;

				s#/># >#;
			}

			s/^t://;
			s/^(.*?)>/$1 >/;

			$self -> _dmesg( "Value is: $_", 3 );
			( $Tag, $Rest, $Data ) = $_ =~ /^(\S+)\s*(.*?)>(.*)$/s;
			# OK, do the push/pop hip/hop
			if( $ShouldPush )
			{
				$ThisNode = $ThisNode -> newchild();

				$ThisNode -> Type( 'TAG' );
				$ThisNode -> Data( Site::Template::Element -> new() );
				$ThisNode -> Data() -> Tag( $Tag );
				$ThisNode -> Data() -> parse( $Rest );
			}

			if( $ShouldPop )
			{
				if( $Tag ne $ThisNode -> Data() -> Tag() )
				{
					$self -> _dfatal( "Got: " . $Tag . 
						" expected: " . 
						$ThisNode -> Data() -> Tag() . 
						"($_)" );
				}

				$ThisNode = $ThisNode -> getparent();
			}
		}
		else
		{
			# Just data, put it in a data object
			$Data = "<" . $_;
		}

		# Crush the data?
		if( ( $self -> Crush() ) && ( $ThisNode -> Data() -> Tag() ne "nocrush" ) )
		{
			$Data =~ s/ +/ /g;
			# Also purge all ws up to nl
			$Data =~ s/\s+\n/\n/gm;

			if( $CrushNL )
			{
				$Data =~ s/\n+/\n/gs;
			}
			$Data =~ s/\s*\n\s*$/ /gs;
		}

		# Add a data block
		if( ( $ThisNode -> HasChildren() ) &&
		( $ThisNode -> Children() -> [ -1 ] -> Type() eq "DATA" ) )
		{
			$ThisNode = $ThisNode -> Children() -> [ -1 ];
			$ThisNode -> add( $Data );
		}
		else
		{
			$ThisNode = $ThisNode -> newchild();
			$ThisNode -> Type( 'DATA' );
			$ThisNode -> Data( $Data );
		}

		$ThisNode = $ThisNode -> getparent();

		# Should be OK to crushnl now
		$CrushNL = 1;
	}

	# Save my cache?
	if( $self -> SaveCache() )
	{
		open( OUT, ">" . $self -> SaveCache() );
		print OUT Dumper( $TopLevel );
		close( OUT );
	}

	# OK, now starting with TOPLEVEL, process each set of children
	# starts with top level variables
	$self -> _PROCESSTREE( $TopLevel, $self -> Objects() );
}

sub _PROCESSTREE
{
	# Takes a DocTree
	my $self = shift;
	my $DocTree = shift;
	my $Vars = shift;

	my( $SaveVars, $RowRef, $OutputData );
	my( $Obj, $ChildProcd, $Val, $LoopCount, $TempTempl, $TempVal );

	# Do what I want
	$ChildProcd = 0;
	if( $DocTree -> Type() eq "DATA" )
	{
		if( $self -> DecodeEntities() )
		{
			$OutputData = decode_entities( $DocTree -> Data() );
		}
		else
		{
			$OutputData = $DocTree -> Data();
		}

		if( $self -> Print() )
		{
			print $OutputData;
		}
		$self -> { OUTPUT } .= $OutputData;
	}elsif( $DocTree -> Type() eq "TAG" )
	{
		# A tag object
		$Obj = $DocTree -> Data();
		$self -> _dmesg( "TAG IS: " . $Obj -> Tag(), 4 );
		$self -> _dmesg( "OBJ IS: " . ref( $Obj ), 4 );
		if( $Obj -> Tag() eq "var" )
		{
			$self -> _dmesg( "NAME is " . $Obj -> get( "name" ), 4 );
			$Val = $self -> __VAR( $Obj, $Vars );


			if( $Obj -> get( "display" ) ne "no" )
			{
				if( $self -> DecodeEntities() )
				{
					$OutputData = decode_entities( $Val );
				}
				else
				{
					$OutputData = $Val;
				}

				if( $self -> Print() )
				{
					print $OutputData;
				}
				$self -> { OUTPUT } .= $OutputData;
			}
		}elsif( $Obj -> Tag() eq "iftrue" )
		{
			# If the value of name is false then short out children
			if( ! $self -> __VAR( $Obj, $Vars ) )
			{
				$ChildProcd = 1;
			}
		}elsif( $Obj -> Tag() eq "iffalse" )
		{
			# If the value of name is true then short out children
			if( $self -> __VAR( $Obj, $Vars) )
			{
				$ChildProcd = 1;
			}
		}elsif( $Obj -> Tag() eq "loop" )
		{
			# Looping, shift in to var names variable space.
			$SaveVars = $Vars;
			#$RowRef = $Vars -> { $Obj -> get( "name" ) };
			$RowRef = $self -> __VAR( $Obj, $Vars );

			if( ! defined( $RowRef ) )
			{
				$self -> _dmesg( "Undefined row referenced: " . $Obj -> get( "name" ), 3 );
			}

			$LoopCount = 1;
			for ( @{ $RowRef } )
			{
				# Get varspace, recurse
				$Vars = $_;

				# If not a ref of some sort, make it one
				# And push the value in to $_ in the ref
				if( ! ref( $Vars ) )
				{
					$Vars = { __TMPLVAL => $Vars };
				}

				# For lmod and lcount
				$Vars -> { __TMPLOOP } = $LoopCount++;
				# For scoping
				$Vars -> { __TMPLPARENT } = $SaveVars;

				for ( @{ $DocTree -> Children() } )
				{
					$self -> _PROCESSTREE( $_, $Vars );
				}
			}

			$Vars = $SaveVars;

			$ChildProcd = 1;
		}elsif( $Obj -> Tag() eq "with" )
		{
			# Similar to the VB with statement
			# Get varspace, recurse
			$SaveVars = $Vars;
			$Vars = $self -> __VAR( $Obj, $Vars );

			$Vars -> { __TMPLPARENT } = $SaveVars;
			for ( @{ $DocTree -> Children() } )
			{
				$self -> _PROCESSTREE( $_, $Vars );
			}
			$Vars = $SaveVars;

			$ChildProcd = 1;
		}elsif( $Obj -> Tag() eq "set" )
		{
			# Set a variable in the current variable scope
			$self -> __SETVAR( $Obj, $Vars );
		}elsif( $Obj -> Tag() eq "include" )
		{
			$TempTempl = Site::Template -> new();
			$TempVal = $Obj -> get( 'name' );
			if( $TempVal )
			{
				$TempVal = $self -> __VAR( $Obj, $Vars );
			}else
			{
				$TempVal = $Obj -> get( 'file' );
			}
			$TempTempl -> load( $TempVal );
			$TempTempl -> registerdefault( $Vars );
			$TempTempl -> print( $self -> print() );
			$TempTempl -> display();

			# If not printing, put in output
			$self -> { OUTPUT } .= $TempTempl -> output();
		}elsif( $Obj -> Tag() eq "lcount" )
		{
			# Check if loop count equal to val
			# If not equal, short off our children
			if( $Vars -> { __TMPLOOP } != 
				$Obj -> get( "value" ) )
			{
				$ChildProcd = 1;
			}
		}elsif( $Obj -> Tag() eq "lmod" )
		{
			# Check if loop count mod val is 0
			# If not equal, short off our children
			# Mostly useful for making tables of N
			# per cell
			if( $Vars -> { __TMPLOOP } % 
				$Obj -> get( "value" )  != 0 )
			{
				$ChildProcd = 1;
			}
		}elsif( $Obj -> Tag() eq "nul" )
		{
			# Null tag, do nothing
		}elsif( $Obj -> Tag() eq "nocrush" )
		{
			# Do nothing, it was just not crushed
		}else
		{
			$self -> _dmesg( "Unknown tag: " . $Obj -> Tag(), -1 );
		}
	}

	if( ! $ChildProcd )
	{
		for ( @{ $DocTree -> Children() } )
		{
			$self -> _PROCESSTREE( $_, $Vars );
		}
	}
}

sub __VAR
{
	my $self = shift;
	my $VarObj = shift();
	my $Orig = $VarObj -> get( 'name' );
	my $Obj = shift;

	my $EscapeType = lc( $VarObj -> get( 'escape' ) );

	my( $ThisStep, $NextStep, $PreArg, @Args );

	# Do the actual decoding
	$ThisStep = $self -> __VAR_ACTUALDECODE( $Orig, $Obj );

	if( $EscapeType eq "html" )
	{
		$ThisStep = Site::Page -> encode_entities( $ThisStep );
	}elsif( $EscapeType eq "uri" )
	{
		$ThisStep = Site::Page -> escape_uri( $ThisStep );
	}elsif( $EscapeType eq "custom" )
	{
		$ThisStep = &{ $self -> custom_escape() }( $ThisStep );
	}elsif( $EscapeType ne "none" )
	{
		$self -> _dmesg( "Unknown escape type: $EscapeType", -1 );
	}

	if( $VarObj -> get( "convert" ) =~ /nl2br/ )
	{
		$ThisStep =~ s/\n/<BR>/g;
	}

	if( $VarObj -> get( "sformat" ) )
	{
		$ThisStep = sprintf( $VarObj -> get( "sformat" ), $ThisStep );
	}
	
	return( $ThisStep );
}

sub __VAR_ACTUALDECODE
{
	my $self = shift;
	my $Orig = shift;
	my $Obj = shift;

	my( $ThisStep, $NextStep, $PreArg, @Args );
	$self -> _dmesg( "Asked to dereference: '$Orig'", 4 );

	# Perform indirection
	while( $Orig =~ /^\$/ )
	{
		$self -> _dmesg( "Indir: $Orig", 2 );
		$Orig =~ s/^.//;
		$Orig = $self -> __VAR_ACTUALDECODE( $Orig, $Obj );
	}

	my @Steps = split( /:/, $Orig );

	# Decode what's left...
	$ThisStep = $Obj;
	for ( @Steps )
	{
		if( /\(/ )
		{
			# Do arg proc
			( $NextStep, $PreArg ) = $_ =~ /^(.*)\((.*)\)/;
			@Args = split( /,/, $PreArg );
		}
		else
		{
			$NextStep = $_;
			@Args = ();
		}

		$self -> _dmesg( "Next step: '$NextStep'", 4 );
		# .. means go up one level of scope
		if( $NextStep !~ /^\!/ )
		{
			if( ref( $ThisStep ) ne "HASH" )
			{
				$ThisStep = $ThisStep -> $NextStep( @Args );
			}
			else
			{
				$ThisStep = $ThisStep -> { $NextStep };
			}
		}
		else
		{
			if( $NextStep eq '!..' )
			{
				$ThisStep = $ThisStep -> { __TMPLPARENT };
			} elsif( $NextStep eq '!loop' )
			{
				$ThisStep = $ThisStep -> { __TMPLOOP };
			} elsif( $NextStep eq '!_' )
			{
				$ThisStep = $ThisStep -> { __TMPLVAL };
			}
		}
	}
	$self -> _dmesg( "Dereference complete: '$Orig'", 4 );

	return( $ThisStep );
}

sub __SETVAR
{
	my $self = shift;
	my $Obj = shift;

	my $Vars = shift;

	$Vars -> { $Obj -> get( "name" ) } = $Obj -> get( "value" );
}

sub addlib
{
	shift;
	push( @Libs, shift() );
}

sub setlibs
{
	shift;
	@Libs = @_;
}

sub register_reset
{
	shift() -> { OBJECTS } = {};
}

sub register_default
{
	shift() -> registerdefault( @_ );
}

sub resolvefile
{
	my $self = shift;
	my $File = shift;

	for ( @Libs )
	{
		if( stat( $_ . "/" . $File ) )
		{
			return( abs_path( $_ ) . "/" . $File );
		}
	}

	# Convert to fully qualified path...
	die( "File doesn't exist?? $File" );
	return( undef );
}

1;

__END__

=head1 NAME

Site::Template - magical template library, mostly for html
	but useful for most any text processing

=head1 SYNOPSIS

        use Site::Template;
	$t = Site::Template -> new();
	$t -> load( "mytemplate.tmpl" );
	$t -> register( Something => $s,
			Else => $e );
	$t -> display();

=head1 DESCRIPTION

The Site::Template library allows simple text expansion.  Mostly useful
for websites or other text-oriented jobs.

	new		Return a new empty template object
	register	Register objects and names with the template
	registerdefault	Takes whatever the arg was and makes it the
			current namespace in the template.  This
			means if it was an object the methods are
			directly accessable and if it's a hashref
			that you don't need to redirect.
	crush		Set to 1 or 0, if set to 1 it will auto-eliminate
			white space in the output document.
	print		Set to 1 or 0, if false it will generate output
			to the Output() method.
	decodeentities	If set to true it will automatically decode
			all &*; entities while generating output,
			this could be used to generate other types
			of text files than HTML and allow escaping
			the control characters.
	load		Load a template from disk.  If there is a
			./cache directory, it will check to see
			if the cached version is newer than the
			template to load, if so it will use the cached
			version.  This is just a Data::Dumper printout
			that can be loaded, that allows the template
			to skip re-parsing the on-disk template file for
			keywords.
	display		Display the template output, unless the Print
			flag is set to false, then it will generate the
			output into the Output member and also return
			if on this call.
	addlib		Adds a library to the list of template libraries.
			By default the list includes "." and "Site".  You
			can add as many libs as you need, the file used
			will be the first file that matches the name for
			the load function in the library list.
	setlibs		Takes an array and overwrites the entire lib list
			with whatever is passed.

=head1 TEMPLATES

Templates are just plain text files, though they will usually contain HTML
and other markup for the template.  All template directives are called by
using <t:*/> syntax.  Notice that if there is no closing tag that you must
include the /> like with XML.

	<t:var name=NAME/>
		Output the value of a variable bound to NAME.
		Object methods can be referenced using ':'s, like

	<t:var name=MyObject:Foo:Bar/>

		Is essentially the same as $MyObject -> Foo() -> Bar();
		You can also specify an optional escape style:

		escape=html	Encode HTML entities
		escape=uri	Encode as URI
		escape=custom	Use custom escape handler

		If an object method prints something and you don't
		want the return value of the method printed,
		set display=no.

		If you prefix the name with a $, it performs a level
		of indirection.  Like if you have $foo, and foo=bar,
		then you get the value of bar not foo.

	<t:loop name=NAME></t:loop>
		Loops through the arrayref called NAME.  The values of
		the arrayref should be hashrefs, for example:

	$t -> register( names => [
					{
						first => 'Jeff',
						last => 'Jeffries'
					}, {
						first => 'Bob',
						last => 'Barker'
					}
				] );

		And the template code to reference it might look like:

	<t:loop name=names>
	First name: <t:var name=first/>
	Last name: <t:var name=last/>
	</t:loop>

		Any other t:* directives can be nested inside the t:loop

		Within a loop there are some special variables.  !loop
		is the current loop count, !.. will take you back
		a namespace level, and !_ is the current value.

	<t:with name=object></t:with>

		Shifts the namespace.  For example, if you have
		registered:

		{
			colors =>
				{
					red => 'ff0000',
					blue => '0000ff'
				}
			default => 'ff00ff'
		}

		You can:
	<t:with name=colors>
	Red is <t:var name=red/> and blue is <t:var name=blue/>
	</t:with>
	Default is <t:var name=default/>

	<t:set name=variable value=newvalue/>

		Sets a variable in the current scope.  You should
		avoid this if possible.  Better to set the code in
		the script.

	<t:lcount value=when></t:lcount>

		Runs the code when loopcount is equal to 'when'.  Count
		starts from 1.

	<t:lmod ></t:lmod>

		Shows the code in >< if the current loop count mod N
		if equal to 0.  Good for making every other row of
		a table grey, etc.

=head1 AUTHOR

Copyright (c) 2001 Steve Slaven <bpk@hoopajoo.net> All rights reserved.

This program is free software  and is provided "as is" without
express or implied warranty. You can redistribute it and/or
modify it under the same terms as Perl itself.

=cut

